/* gxTar - Gnomified Archive Frontend
 * Copyright (C) 1998 Chris Rogers
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option) 
 * any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 */


#include "archive.h"
#include "error.h"
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "statusbar.h"

/* File Stuff */


#define KNOWNEXT  4 

#define COLNUM 6

struct archive_type {

  gchar *ext;
  gchar *compress;
  gchar *uncompress;
  gchar *view;
  gchar *extract;
  gchar *create;
  gchar *add;
  gchar *delete;

  int maxcolumns;
  int namecolumn;
  int *order; 
  int skip;
  int (*readit) (FILE *, int);
  int (*extractit)(gchar *);
  int (*addit)(gchar *); 
  int (*deleteit)(void); 

};


int tar_read(FILE *fl, int found );
int gz_extract(gchar *dir);
int tar_extract(gchar *dir);
int zip_extract(gchar *dir);


int tar_add(gchar *fle);
int tgz_add(gchar *fle);
int zip_add(gchar *fle);

int tar_delete();
int tgz_delete();
int zip_delete();


static struct archive_type archList[KNOWNEXT] =
{
  {".gz","gzip","gunzip","-lv","-x","-c","-a",NULL,9,8,NULL,1,tar_read,gz_extract,NULL,NULL},
#ifdef __hpux
  {".tgz","tar","tar","-tvf","-xvf","-cf","-rf","--delete",6,5,NULL,-1,tar_read,tar_extract,tgz_add,tgz_delete},
#else
  {".tgz","tar","tar","tvzf","-xzf","-czf","-rf","--delete",6,5,NULL,-1,tar_read,tar_extract,tgz_add,tgz_delete},
#endif
  {".tar","tar","tar","tvf","-xf","-cf","-rf","--delete",6,5,NULL,-1,tar_read,tar_extract,tar_add,tgz_delete},
  {".zip","zip","unzip","-Z","-x","-c","-a","-d",9,8,NULL,1,tar_read,zip_extract, zip_add,zip_delete}

};


static int didInit = FALSE;

/*not sure how to avoid this*/
void order_init() {

  if (didInit == FALSE) {
    didInit = TRUE;
    archList[0].order = g_malloc(sizeof(int)*9);
    archList[0].order[0] = -1;
    archList[0].order[1] = -1;
    archList[0].order[2] = 1;
    archList[0].order[3] = -1;
    archList[0].order[4] = -1;
    archList[0].order[5] = -1;
    archList[0].order[6] = 2;
    archList[0].order[7] = 4;
    archList[0].order[8] = 0;

    
    archList[1].order = g_malloc(sizeof(int)*6);
    archList[1].order[0] = 3;
    archList[1].order[1] = -1;
    archList[1].order[2] = 2;
    archList[1].order[3] = 1;
    archList[1].order[4] = -1;
    archList[1].order[5] = 0;
    
    archList[2].order = g_malloc(sizeof(int)*6);
    archList[2].order[0] = 3;
    archList[2].order[1] = -1;
    archList[2].order[2] = 2;
    archList[2].order[3] = 1;
    archList[2].order[4] = -1;
    archList[2].order[5] = 0;
    
    archList[3].order = g_malloc(sizeof(int)*8);
    archList[3].order[0] = 3;
    archList[3].order[1] = -1;
    archList[3].order[2] = -1;
    archList[3].order[3] = 2;
    archList[3].order[4] = -1;
    archList[3].order[5] = -1;
    archList[3].order[6] = 1;
    archList[3].order[7] = -1;
    archList[3].order[8] = 0;
    }
}


/*these guys get an open file descriptor and fill the file list*/

int tar_read(FILE *fl,int found) {
  gchar *line, *holdline, *tmp; /*used to do the parsing*/
  gchar *text[COLNUM];
  int i,j; 
  int pos;
  int maxitems = archList[found].maxcolumns;
  int *places  = archList[found].order;
  gchar *tmpstr1, *slash;
  int sz;


  for (i = 0; i < COLNUM; i++) text[i] = NULL;

  line = g_malloc(1024);
  holdline = line;

  for  (i = 0; i < archList[found].skip; i++) fgets(line, 1023, fl);
  j = 0;
  fgets(line, 1023, fl);
  while (!feof(fl)) {

    for (pos= 0; pos < maxitems; pos++) {
      while ((tmp = strchr(line, ' ')) == line) line = line +1;
      if (places[pos] != -1) { /*skip -1*/
	if (pos == (maxitems-1)) {
	  if ((tmp = strchr(line, '\n')) == NULL) { 
	    /*g_free(holdline); */
	    for (i = 0; i < COLNUM; i++) 
	      if (text[i] != NULL) 
		{g_free(text[i]);  text[i] = NULL;}
	    break; /* return FALSE; */
	  }
	}
	else {
	  if ((tmp = strchr(line, ' ')) == NULL) { 
	    /*g_free(holdline);   */
	    for (i = 0; i < COLNUM; i++) 
	        if (text[i] != NULL) { g_free(text[i]);  text[i] = NULL;} 
	    break; /*return FALSE; */
	  } 
	}

	/*do something special if pos = namecolumn*/
	if (pos ==  archList[found].namecolumn) {
	  tmpstr1 = g_malloc(tmp - line +1);
	  strncpy(tmpstr1, line, tmp - line);
	  tmpstr1[tmp - line] = '\0';
	  
	  slash = strrchr(tmpstr1, '/');
  
	  if (slash == tmpstr1 + strlen(tmpstr1) -1) { /*do directory stuff*/

	    slash[0] = '\0';

	    slash = strrchr(tmpstr1, '/');

	    if (slash == NULL) { /*base directory*/

	      text[0] = g_malloc( strlen(tmpstr1) + 2); 
	      strcpy(text[0], tmpstr1);
	      text[0][strlen(tmpstr1)] = '/';
	      text[0][strlen(tmpstr1)+1] = '\0';

	      text[COLNUM-1] = g_malloc( 2 );
	      strcpy(text[COLNUM-1]," ");
	      text[COLNUM-1][1] = '\0';
	    }
	    else {
	      
	      text[0] = g_malloc( (int)  tmpstr1 + strlen(tmpstr1) -  (int) slash + 1);
	      strcpy(text[0], slash+1);
	      text[0][(int) tmpstr1 + strlen(tmpstr1) -  (int) slash - 1] = '/';
	      text[0][(int) tmpstr1 + strlen(tmpstr1) -  (int) slash ] = '\0';
	      sz = strlen(tmpstr1) - strlen(text[0])+1;
	      text[COLNUM-1] = g_malloc(sz);
	      strncpy(text[COLNUM-1], tmpstr1,sz-1);
	      text[COLNUM-1][sz - 1] = '\0';
	      
	      
	    }
	    
	  }
	  else { /*do file stuff*/

	    if (slash == NULL) {
	      
	      text[0] = g_malloc(strlen(tmpstr1)+1);
	      strcpy(text[0], tmpstr1);
	      text[0][strlen(tmpstr1)] = '\0';

	      text[COLNUM-1] = g_malloc( 2 );
	      strcpy(text[COLNUM-1]," ");
	      text[COLNUM-1][1] = '\0';

	    }
	    else{
	      
	      text[0] = g_malloc( (int)  tmpstr1 + strlen(tmpstr1) -  (int) slash);
	      strcpy(text[0], slash+1);
	      text[0][(int) tmpstr1 + strlen(tmpstr1) -  (int) slash - 1] = '\0';
	      
	      sz = strlen(tmpstr1) - strlen(text[0])+1;
	      text[COLNUM-1] = g_malloc(sz);
	      strncpy(text[COLNUM-1], tmpstr1,sz-1);
	      text[COLNUM-1][sz - 1] = '\0';
	      
	    }
	  }
	  g_free(tmpstr1);tmpstr1=NULL;
	}
	else {
	  text[places[pos]] = g_malloc(tmp - line +1);
	  strncpy(text[places[pos]], line, tmp - line);
	  text[places[pos]][tmp - line] = '\0';
	}
      }
      line = tmp +1;
    
      /*move past any consequtive spaces */
      
    }
    if (text[0] != NULL)
      {
	gtk_clist_append( GTK_CLIST (arch->listFiles) , text);
	j++;
      }
    
    for (i = 0; i < COLNUM; i++) if (text[i] != NULL) {g_free(text[i]); text[i] = NULL;} 

    line = holdline;
    fgets(line, 1023, fl);
  } 
  g_free(line);


 
  if (strcmp(archList[found].uncompress,"unzip") == 0)
    gtk_clist_remove (GTK_CLIST (arch->listFiles),j-1);
    

  /* until the Gnome version catches up to gtk 1.1.3 */
  /*
    for (i = 0; i < COLNUM; i++) {
    gtk_clist_set_column_visibility (GTK_CLIST (arch->listFiles),i,FALSE);
    
    gtk_clist_set_column_width (GTK_CLIST (arch->listFiles),i,gtk_clist_optimal_column_width (GTK_CLIST (arch->listFiles),i));
    }
  
    for (j = 0; j < archList[found].maxcolumns; j++) 
    if (archList[found].order[j] != -1) 
    gtk_clist_set_column_visibility (GTK_CLIST (arch->listFiles),archList[found].order[j],TRUE);
  */

  return TRUE; 
}


void file_selection_hide_fileops (GtkWidget *widget, GtkFileSelection *fs) {
  gtk_file_selection_hide_fileop_buttons (fs);
}

void file_selection_ok (GtkWidget *widget, GtkFileSelection *fs) {

  if (arch->OpenFile != NULL) {g_free(arch->OpenFile);  arch->OpenFile = NULL;}
  arch->OpenFile = g_malloc0(strlen(gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)))+1);
  strcpy(arch->OpenFile, gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));

  gtk_object_destroy (GTK_OBJECT (fs)); /*or widget*/

  openArchive();
}


void create_file_selection () {
  static GtkWidget *window = NULL;
  GtkWidget *button;
  
  if (!window)
    {
      window = gtk_file_selection_new ("Open Archive");
      
      gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (window));
      
      gtk_window_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
      
      gtk_signal_connect (GTK_OBJECT (window), "destroy",
                          GTK_SIGNAL_FUNC(gtk_widget_destroyed),
                          &window);

      gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (window)->ok_button),
                          "clicked", GTK_SIGNAL_FUNC(file_selection_ok),
                          window);
      gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (window)->cancel_button),
                                 "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
                                 GTK_OBJECT (window));
      
      button = gtk_button_new_with_label ("Hide Fileops");
      gtk_signal_connect (GTK_OBJECT (button), "clicked",
                          (GtkSignalFunc) file_selection_hide_fileops, 
                          (gpointer) window);
      gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (window)->action_area), 
                          button, FALSE, FALSE, 0);
      gtk_widget_show (button);
      
      button = gtk_button_new_with_label ("Show Fileops");
      gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                                 (GtkSignalFunc) gtk_file_selection_show_fileop_buttons, 
                                 (gpointer) window);
      gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (window)->action_area), 
                          button, FALSE, FALSE, 0);
      gtk_widget_show (button);
    }

  
  if (!GTK_WIDGET_VISIBLE (window))
    {
      gtk_widget_show (window);
      gtk_grab_add(window);
    }
  else
    {
      gtk_widget_destroy (window);
      gtk_grab_remove(window);
    }
}
 

void extract_selection_hide_fileops (GtkWidget *widget, GtkFileSelection *fs) {
  gtk_file_selection_hide_fileop_buttons (fs);
}

void extract_selection_ok (GtkWidget *widget, GtkFileSelection *fs) {

  gchar *dir;

  g_print ("%s\n", gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));

  dir = g_malloc0(strlen(gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)))+1);

  strcpy(dir, gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));

  gtk_widget_destroy (GTK_WIDGET (fs));
  
  archList[arch->filetype].extractit(dir);

}


void create_extract_selection () {
  static GtkWidget *window = NULL;
  GtkWidget *button;
  
  if (!window)
    {
      window = gtk_file_selection_new ("Extract Archive to..");
      
      gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (window));
      
      gtk_window_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
      
      gtk_signal_connect (GTK_OBJECT (window), "destroy",
                          GTK_SIGNAL_FUNC(gtk_widget_destroyed),
                          &window);

      gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (window)->ok_button),
                          "clicked", GTK_SIGNAL_FUNC(extract_selection_ok),
                          window);
      gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (window)->cancel_button),
                                 "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
                                 GTK_OBJECT (window));
      
      button = gtk_button_new_with_label ("Hide Fileops");
      gtk_signal_connect (GTK_OBJECT (button), "clicked",
                          (GtkSignalFunc) extract_selection_hide_fileops, 
                          (gpointer) window);

      gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (window)->action_area), 
                          button, FALSE, FALSE, 0);
      gtk_widget_show (button);
      
      button = gtk_button_new_with_label ("Show Fileops");
      gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                                 (GtkSignalFunc) gtk_file_selection_show_fileop_buttons, 
                                 (gpointer) window);

      gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (window)->action_area), 
                          button, FALSE, FALSE, 0);
      gtk_widget_show (button);
    }

  gtk_widget_hide(GTK_FILE_SELECTION (window)->file_list);
  
  if (!GTK_WIDGET_VISIBLE (window))
    {
      gtk_widget_show (window);
      gtk_grab_add(window);
    }
  else
    {
      gtk_widget_destroy (window);
      gtk_grab_remove(window);
    }
}
void add_selection_hide_fileops (GtkWidget *widget, GtkFileSelection *fs) {
  gtk_file_selection_hide_fileop_buttons (fs);
}


void add_selection_ok (GtkWidget *widget, GtkFileSelection *fs) {

  gchar *fle;

  g_print ("%s\n", gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));

  fle = g_malloc0(strlen(gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)))+1);

  strcpy(fle, gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));

  gtk_widget_destroy (GTK_WIDGET (fs));
  
  /*  add filename (fle) */
  archList[arch->filetype].addit(fle);

}


void create_add_selection () {
  static GtkWidget *window = NULL;
  GtkWidget *button;
  
  if (!window)
    {
      window = gtk_file_selection_new ("Add file to archive");
      
      gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (window));
      
      gtk_window_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
      
      gtk_signal_connect (GTK_OBJECT (window), "destroy",
                          GTK_SIGNAL_FUNC(gtk_widget_destroyed),
                          &window);

      gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (window)->ok_button),
                          "clicked", GTK_SIGNAL_FUNC(add_selection_ok),
                          window);
      gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (window)->cancel_button),
                                 "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
                                 GTK_OBJECT (window));
      
      button = gtk_button_new_with_label ("Hide Fileops");
      gtk_signal_connect (GTK_OBJECT (button), "clicked",
                          (GtkSignalFunc) add_selection_hide_fileops, 
                          (gpointer) window);

      gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (window)->action_area), 
                          button, FALSE, FALSE, 0);
      gtk_widget_show (button);
      
      button = gtk_button_new_with_label ("Show Fileops");
      gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                                 (GtkSignalFunc) gtk_file_selection_show_fileop_buttons, 
                                 (gpointer) window);

      gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (window)->action_area), 
                          button, FALSE, FALSE, 0);
      gtk_widget_show (button);
    }

 
  if (!GTK_WIDGET_VISIBLE (window))
    {
      gtk_widget_show (window);
      gtk_grab_add(window);
    }
  else
    {
      gtk_widget_destroy (window);
      gtk_grab_remove(window);
    }
}

/*our general clean up and exit routine*/
void quitArchive () {
  gtk_main_quit ();
}


/*select filename, then call openArchive with the filename*/
void openfileArchive() {
  /*prompt for a filename */
  create_file_selection();
}


/*use drag and drop stuff to get a filename*/
void opendndArchive() {
}


/*assumes there is a filename, does the actually open*/
void openArchive() {

  /*for now we are going to work based on file extensions */
  gchar *ext;
  gint i;
  gint found = -1;
  gchar *tmpOut;
  gchar *savedot;
  gchar cmd[1024];
  gint result;
  FILE * INF;
  gchar * title;

  setStatusbar("Opening File...");
  while (gtk_events_pending()) gtk_main_iteration();
  

  /*got get rid of this */
  order_init();

  /*once they have gotten here they have commited to opening a new file, so were gonna wipe the list clean */

  gtk_clist_clear( GTK_CLIST (arch->listFiles));


  /*get the file extension*/
  /*to do this we need to get the bit of the string after the last "." */
  ext = strrchr(arch->OpenFile, '.');
  if ((ext == NULL)||(strlen(ext+1) < 1)){
    if (arch->OpenFile != NULL) { g_free(arch->OpenFile); arch->OpenFile = NULL;}
    ShowError("Not a valid archive!");
    clearStatusbar();
    return;
  }
  
  /*now that we have an extension, so lets see if we understand it*/
  
  for (i = 0; i < KNOWNEXT; i++) 
    if (strcmp(ext, archList[i].ext)==0) found = i;

  /*ok special case cause of .tar.gz, if we find a .gz, lets see if we can find a .tar before it, and then just pretend it was .tgz - eventually I want this behavior to be an option, I prefer it but others might not */


  /* <kludge> */
  if (found != -1)
    if (strcmp(ext, ".gz")==0) { /*ok, we are a .gz */
      savedot = ext;
      ext[0] = '\0'; /*knock off the end of the filename*/
      if ((ext = strrchr(arch->OpenFile, '.')) != NULL) /*look for the next to last .*/
	if (strcmp(ext, ".tar")==0)  /*look we found a tar*/
	  	  for (i = 0; i < KNOWNEXT; i++) if (strcmp(".tgz",archList[i].ext )==0) found = i; /*if it is not there something is wrong*/
      savedot[0] = '.'; /* put it back */
    } /*we leave this with a "ficticous" found, but one that will do what we want */
  /*</kludge> */


  if (found ==-1) {
    if (arch->OpenFile != NULL) {g_free(arch->OpenFile); arch->OpenFile = NULL;}
    clearStatusbar();
    ShowError("Not a valid archive!");
    return;
  }

  /*  g_print("using %s to unencrypt\n",archList[found].uncompress);  */

  /*hmm - using pipes would probably be smarter*/
  /*also this is all kinds of broken because of the fixed size cmd */
  tmpOut = tempnam("","gxtar");
  sprintf(cmd, "%s %s %s > %s", archList[found].uncompress, archList[found].view,arch->OpenFile, tmpOut); 

  /*g_print("%s\n",cmd);*/
  result = system(cmd);
  /*  g_print("result:%d\n",result); */

  if (result != 0) { /*problem execing*/
    if (arch->OpenFile != NULL) {g_free(arch->OpenFile); arch->OpenFile = NULL;}
    clearStatusbar();
    ShowError("Error opening archive");
    return;
  }

  if ((INF = fopen(tmpOut,"r")) == NULL) {
    if (arch->OpenFile != NULL) {g_free(arch->OpenFile); arch->OpenFile = NULL;}
    clearStatusbar();
    ShowError("Error opening archive");
    return;
  }
  
  gtk_clist_freeze( GTK_CLIST(arch->listFiles));
  
  archList[found].readit(INF,found);
  
  gtk_clist_thaw( GTK_CLIST (arch->listFiles));
  arch->filetype = found;
  
  title = g_malloc(sizeof(gchar)*(8+strlen(arch->OpenFile)+1));
  strcpy(title, "gxTar - ");
  strcat(title, arch->OpenFile);
  gtk_window_set_title (GTK_WINDOW (window), title);

  fclose(INF);
  unlink(tmpOut);
  clearStatusbar();
}

void extractArchive() {

  if (arch->OpenFile != NULL)
    create_extract_selection();
}

int  gz_extract(gchar * dir){return -1;}

int  zip_extract(gchar * dir){

  GList *selection;
  gint index;
  gchar *text;
  gchar cmd[1024];
  gint result;
  

  selection = GTK_CLIST(arch->listFiles)->selection;

  
  if (selection == NULL) {
    sprintf(cmd, "%s %s %s -d %s", archList[arch->filetype].uncompress, archList[arch->filetype].extract,arch->OpenFile,dir);
  }else{
    sprintf(cmd,"%s %s -d %s %s", archList[arch->filetype].uncompress, archList[arch->filetype].extract, dir, arch->OpenFile);

    while(selection) {
      index = (gint) selection->data;
      strcat(cmd," ");
      gtk_clist_get_text(GTK_CLIST(arch->listFiles), index, COLNUM-1, &text);
      strcat(cmd,text); 
      gtk_clist_get_text(GTK_CLIST(arch->listFiles), index, 0, &text);
      strcat(cmd,text); 
      selection= selection->next;
    }
  
  }
  
  g_print("%s\n",cmd);
  result = system(cmd); 
  
  return result;
}

int  tar_extract(gchar * dir){

  GList *selection;
  gint index;
  gchar *text;
  gchar *tmpOut;
  gchar cmd[1024];
  gint result;
  FILE * src;
  

  selection = GTK_CLIST(arch->listFiles)->selection;
  tmpOut = NULL;
  
  if (selection == NULL) {
    sprintf(cmd, "cd %s; %s %s %s ",dir, archList[arch->filetype].uncompress, archList[arch->filetype].extract,arch->OpenFile);
  }else{
    tmpOut = tempnam("","gxtar");
    src = fopen(tmpOut,"w");
    
    while(selection) {
      index = (gint) selection->data;
      gtk_clist_get_text(GTK_CLIST(arch->listFiles), index, COLNUM-1, &text);
      fputs(text,src); 
      gtk_clist_get_text(GTK_CLIST(arch->listFiles), index, 0, &text);
      fputs(text,src); 
      fputc('\n',src);

      selection= selection->next;
    }
    fclose(src);
    sprintf(cmd, "cd %s; %s %s %s -T %s", dir, archList[arch->filetype].uncompress, archList[arch->filetype].extract,arch->OpenFile, tmpOut);
  }
  
  g_print("%s\n",cmd);
  result = system(cmd); 
  if (tmpOut != NULL) unlink(tmpOut);  

  return result;
}


void closeArchive(){

  gtk_clist_clear( GTK_CLIST (arch->listFiles));
  if (arch->OpenFile != NULL) {
    g_free(arch->OpenFile); 
    arch->OpenFile = NULL;
  }
  gtk_window_set_title (GTK_WINDOW (window), "gxTar - Unknown");

}

void addArchive(){

  /*check for open archive*/
  /*get file(s) to add */
  /*attempt to add files */
  /*if success, add files to list*/

  if (arch->OpenFile == NULL)
    {
      ShowError("There is no currently open archive.");
      return;
    }

  create_add_selection();
}


void deleteArchive() {

  /*check for open archive*/
  /*get file(s) to delete */
  /*attempt to delete all selected files */
  /*refresh*/


  setStatusbar("Deleting Files...");
  while (gtk_events_pending()) gtk_main_iteration();


  if (arch->OpenFile == NULL)
    {
      ShowError("There is no currently open archive.");
      return;
    }

  /*delete files*/
 archList[arch->filetype].deleteit();
 


  /*hmm this will be really really ugly with large directories, but it makes my life easy for the moment -- lets just reread the file */
  openArchive();
  

}


int tar_add_actual(gchar *src, gchar *fle){

  gchar cmd[1024]; /*this is still horrible*/
  char star[2] = " ";
  int result;
  
  if (fle[strlen(fle)-1] == '/') /*append an * */
    star[0] = '*';
  
  sprintf(cmd, "%s %s %s %s%s",archList[arch->filetype].compress, archList[arch->filetype].add,  src, fle, star);
  g_print("%s\n",cmd);
  result = system(cmd); 
  
  /*what about adding directories? */
  return result;
}



int tar_add(gchar *fle){

  int result;
  setStatusbar("Adding File...");
  while (gtk_events_pending()) gtk_main_iteration();
 
  
  result = tar_add_actual(arch->OpenFile,fle);

  /*hmm this will be really really ugly with large directories, but it makes my life easy for the moment -- lets just reread the file */
  openArchive();
  
  return result; 
}


int zip_add(gchar *fle){
  
  gchar cmd[1024]; /*this is still horrible*/
  char star[2] = " ";
  int result;
  

  setStatusbar("Adding File...");
  while (gtk_events_pending()) gtk_main_iteration();

  if (fle[strlen(fle)-1] == '/') /*append an * */
    star[0] = '*';
  
  sprintf(cmd, "%s %s %s%s",archList[arch->filetype].compress,arch->OpenFile, fle, star);
  g_print("%s\n",cmd);
  result = system(cmd); 
 
  /*what about adding directories? */

  /*hmm this will be really really ugly with large directories, but it makes my life easy for the moment -- lets just reread the file */
  openArchive();
  
  return result;
  
}

int tgz_add(gchar *fle){

  /*ok, since we can't directly add to zipped tars, lets pretend*/
  gchar *tmpOut;
  gchar cmd[1024]; /*this is still horrible*/
  int result;
  tmpOut = tempnam("","gxtar");

  sprintf(cmd, "gunzip -c  %s > %s",arch->OpenFile, tmpOut);
  g_print("%s\n",cmd);
  result = system(cmd); 
  
  
  tar_add_actual(tmpOut,fle);
  
  sprintf(cmd, "gzip %s",tmpOut);
  g_print("%s\n",cmd);
  result = system(cmd); 

  sprintf(cmd, "mv %s.gz %s",tmpOut, arch->OpenFile);
  g_print("%s\n",cmd);
  result = system(cmd); 


  unlink(tmpOut);
  sprintf(cmd,"%s.gz",tmpOut);
  unlink(cmd);
  
  /*hmm this will be really really ugly with large directories, but it makes my life easy for the moment -- lets just reread the file */
  openArchive();

  return 1;

}




int tar_delete_actual(gchar *src, gchar *fle) {

  GList *selection;
  gint index;
  gchar *text;
  gchar cmd[1024];
  gint result;
  
  
  selection = GTK_CLIST(arch->listFiles)->selection;
  
  
  if (selection == NULL) {
    ShowError("No files selected!");
    clearStatusbar();
    return -1;
  }else{
    sprintf(cmd,"%s %s -f %s", archList[arch->filetype].compress, archList[arch->filetype].delete, src);
    
    while(selection) {
      index = (gint) selection->data;
      strcat(cmd," ");
      gtk_clist_get_text(GTK_CLIST(arch->listFiles), index, COLNUM-1, &text);
      strcat(cmd,text); 
      gtk_clist_get_text(GTK_CLIST(arch->listFiles), index, 0, &text);
      strcat(cmd,text); 
      selection= selection->next;
    }
    
  }
  
  g_print("%s\n",cmd);
  result = system(cmd); 
  
  return result;
}


int tar_delete(gchar *fle){

  return tar_delete_actual(arch->OpenFile, fle);

}


int tgz_delete(gchar *fle){

 /*ok, since we can't directly delete from  zipped tars, lets pretend*/
  gchar *tmpOut;
  gchar cmd[1024]; /*this is still horrible*/
  int result;
  tmpOut = tempnam("","gxtar");

  sprintf(cmd, "gunzip -c  %s > %s",arch->OpenFile, tmpOut);
  g_print("%s\n",cmd);
  result = system(cmd); 
  
  
  result = tar_delete_actual(tmpOut,fle);
  
  sprintf(cmd, "gzip %s",tmpOut);
  g_print("%s\n",cmd);
  result = system(cmd); 

  sprintf(cmd, "mv %s.gz %s",tmpOut, arch->OpenFile);
  g_print("%s\n",cmd);
  result = system(cmd); 


  unlink(tmpOut);
  sprintf(cmd,"%s.gz",tmpOut);
  unlink(cmd);

  return 1;

}

int zip_delete(gchar *fle){

  GList *selection;
  gint index;
  gchar *text;
  gchar cmd[1024];
  gint result;
  

  selection = GTK_CLIST(arch->listFiles)->selection;

  
  if (selection == NULL) {
    ShowError("No files selected!");
    clearStatusbar();
    return -1;
  }else{
    sprintf(cmd,"%s %s %s", archList[arch->filetype].compress, archList[arch->filetype].delete, arch->OpenFile);
    
    while(selection) {
      index = (gint) selection->data;
      strcat(cmd," ");
      gtk_clist_get_text(GTK_CLIST(arch->listFiles), index, COLNUM-1, &text);
      strcat(cmd,text); 
      gtk_clist_get_text(GTK_CLIST(arch->listFiles), index, 0, &text);
      strcat(cmd,text); 
      selection= selection->next;
    }
  
  }
  
  g_print("%s\n",cmd);
  result = system(cmd); 
  
  return result;
}
  


void selectAll(){
 gtk_clist_select_all(GTK_CLIST(arch->listFiles));
}

void deselectAll(){
  gtk_clist_unselect_all(GTK_CLIST(arch->listFiles));
}




/*view files*/


void closeView(GtkWidget *widget, gpointer *gp) {

  gtk_widget_destroy (GTK_WIDGET (gp));

}

/*we only view the first selected item*/
void viewFile() {

#ifdef GNOME
  GList *selection;
  GtkWidget *app;
  GtkWidget *less;
  GtkWidget *okbutton;
  gchar cmd[1024];
  gchar fle[1024];
  GtkWidget *box;
  gchar *text;
  int index;

  selection = GTK_CLIST(arch->listFiles)->selection;
  
  if (selection == NULL) { 
    ShowError("No files selected!");
    clearStatusbar();
    return;
  }

  index = (gint) selection->data;
  fle[0] = '\0';
  gtk_clist_get_text(GTK_CLIST(arch->listFiles), index, COLNUM-1, &text);
  strcat(fle,text); 
  gtk_clist_get_text(GTK_CLIST(arch->listFiles), index, 0, &text);
  strcat(fle,text); 
  /* printf("view: %s\n",fle); */

  if (arch->filetype == 1) {
#ifdef __hpux
    sprintf(cmd, "tar -xvf %s %s",  arch->OpenFile, fle);
#else
    sprintf(cmd, "tar -xzOf %s %s",  arch->OpenFile, fle);
#endif
  }else if(arch->filetype == 3) {
    sprintf(cmd, "unzip -p %s %s", arch->OpenFile, fle);
  }

  app = gnome_app_new ("gxTar","View");
  gtk_window_set_modal(GTK_WINDOW(app), TRUE);

  less = gnome_less_new();
  okbutton = gtk_button_new_with_label("Ok");
  box = gtk_vbox_new(FALSE, 0);

  gtk_box_pack_start(GTK_BOX(box), less, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(box), okbutton, FALSE, FALSE, 0);
  gnome_app_set_contents(GNOME_APP(app), box);
  gnome_less_set_fixed_font(GNOME_LESS(less), TRUE);
 
  gtk_signal_connect (GTK_OBJECT (okbutton), "clicked",
		      GTK_SIGNAL_FUNC (closeView), app);

  gtk_widget_show(okbutton);
  gtk_widget_show(less);
  gtk_widget_show(app);
  /*  printf("command: %s\n",cmd); */
  gnome_less_show_command(GNOME_LESS(less),cmd);

#endif /*GNOME*/

}






/*Still to do: */


void newArchive(){

  /*get file name*/
  /*warn if it exists*/
  /*create file*/
  /*set filename in arch->Openfile*/
  
}
